import axinite as ax
from vpython import *
import axinite.tools as axtools
from itertools import cycle
import vpython as vp

colors = cycle([color.red, color.blue, color.green, color.orange, color.purple, color.yellow])

def show(_args, frontend=axtools.vpython_frontend):
    args = _args
    if args.rate is None:
        args.rate = 100
    if args.radius_multiplier is None:
        args.radius_multiplier = 1
    if args.retain is None:
        args.retain = 200

    while args.t < args.limit:
        frontend(args.t, bodies=args.bodies)
        args.t += args.delta


import axinite.tools as axtools
import plotly.graph_objects as go
from itertools import cycle
from astropy.coordinates import CartesianRepresentation

def to_vec(cartesian_representation: CartesianRepresentation):
    return dict(x=cartesian_representation.x.value, y=cartesian_representation.y.value, z=cartesian_representation.z.value)

def plotly_frontend(args: axtools.AxiniteArgs):
    colors = cycle(['red', 'blue', 'green', 'orange', 'purple', 'yellow'])
    global pause
    pause = False

    fig = go.Figure()
    if args.name: fig.update_layout(title=args.name)

    spheres = {}
    labels = {}

    for body in args.bodies:
        body_color = body.color, "plotly" if body.color != "" else next(colors)
        body_retain = body.retain if body.retain != None else args.retain
        spheres[body.name] = go.Scatter3d(x=[body.r[0].x.value], y=[body.r[0].y.value], z=[body.r[0].z.value], mode='markers+lines', marker=dict(size=body.radius.value * args.radius_multiplier, color=body_color))
        labels[body.name] = go.Scatter3d(x=[body.r[0].x.value], y=[body.r[0].y.value], z=[body.r[0].z.value], mode='text', text=[body.name], textposition='top center')
        fig.add_trace(spheres[body.name])
        fig.add_trace(labels[body.name])

    def fn(t, **kwargs):
        for body in kwargs["bodies"]:
            spheres[body.name].x = [body.r[t.value].x.value]
            spheres[body.name].y = [body.r[t.value].y.value]
            spheres[body.name].z = [body.r[t.value].z.value]
            labels[body.name].x = [body.r[t.value].x.value]
            labels[body.name].y = [body.r[t.value].y.value]
            labels[body.name].z = [body.r[t.value].z.value]
        fig.show()
        print(f"t = {t}", end='\r')
        if pause: 
            while pause: pass
            
    return fn



import axinite.tools as axtools
from vpython import *
from itertools import cycle
from astropy.coordinates import CartesianRepresentation
  
def to_vec(cartesian_representation: CartesianRepresentation):
    return vector(cartesian_representation.x.value, cartesian_representation.y.value, cartesian_representation.z.value)
    
def vpython_frontend(args: axtools.AxiniteArgs):
    colors = cycle([color.red, color.blue, color.green, color.orange, color.purple, color.yellow])
    global pause
    pause = False

    scene = canvas()
    scene.select()
    if args.name: scene.title = args.name

    def pause_fn():
        global pause
        pause = not pause

    pause_btn = button(bind=pause_fn, text='Pause', pos=scene.caption_anchor)

    spheres = {}
    labels = {}
    lights = {}

    for body in args.bodies:
        body_color = axtools.string_to_color(body.color, "vpython") if body.color != "" else next(colors)
        body_retain = body.retain if body.retain != None else args.retain
        spheres[body.name] = sphere(pos=to_vec(body.r[0]), radius=body.radius.value * args.radius_multiplier, color=body_color, make_trail=True, retain=body_retain, interval=10)
        labels[body.name] = label(pos=spheres[body.name].pos, text=body.name, xoffset=15, yoffset=15, space=30, height=10, border=4, font='sans')
        if body.light == True: lights[body.name] = local_light(pos=to_vec(body.r[0]), color=body_color)

    
    def fn(t, **kwargs):
        rate(args.rate)
        for body in kwargs["bodies"]:
            spheres[body.name].pos = to_vec(body.r[t.value])
            labels[body.name].pos = spheres[body.name].pos
            try: lights[body.name].pos = spheres[body.name].pos
            except: pass
        print(f"t = {t}", end='\r')
        if pause: 
            while pause: rate(10)
            
    return fn


import axinite as ax
from axinite.tools import AxiniteArgs
import json

def load(args: AxiniteArgs, path: str):
    args.action = lambda t, **kwargs: print(f"Timestep {t} ({((t / args.limit) * 100).value:.2f}% complete)", end="\r")

    bodies = ax.load(*args.unpack(), t=args.t)
    print(f"\nFinished with {len(bodies[0].r)} timesteps")

    if path == "": return bodies
    else: 
        with open(path, 'w+') as f:
            data = {
                "name": args.name,
                "delta": args.delta.value,
                "limit": args.limit.value,
                "t": args.t.value,
                "radius_multiplier": args.radius_multiplier,
                "bodies": []
            }

            for body in bodies: 
                body_data = {
                    "name": body.name,
                    "mass": body.mass.value,
                    "radius": body.radius.value,
                    "r": {k: [v.x.value, v.y.value, v.z.value] for k, v in body.r.items()},
                    "v": {k: [v.x.value, v.y.value, v.z.value] for k, v in body.v.items()}
                }
                if body.color != None:
                    body_data["color"] = body.color
                if body.retain != None:
                    body_data["retain"] = body.retain
                if body.light != None:
                    body_data["light"] = body.light

                data["bodies"].append(body_data)

            if args.radius_multiplier is not None:
                data["radius_multiplier"] = args.radius_multiplier

            if args.rate is not None:
                data["rate"] = args.rate

            if args.retain is not None:
                data["retain"] = args.retain

            json.dump(data, f, indent=4)
            return bodies


import axinite as ax
import axinite.tools as axtools
from vpython import *
from itertools import cycle

colors = cycle([color.red, color.blue, color.green, color.orange, color.purple, color.yellow])

def live(_args: axtools.AxiniteArgs, frontend=axtools.vpython_frontend):
    args = _args
    if args.rate is None:
        args.rate = 100
    if args.radius_multiplier is None:
        args.radius_multiplier = 1
    if args.retain is None:
       args.retain = 200


    args.action = frontend(args)
    ax.load(*args.unpack(), t=args.t)


import astropy.units as u
import axinite as ax
from astropy.coordinates import CartesianRepresentation
import vpython as vp
import numpy as np
import axinite.tools as axtools

def interpret_time(string: str):
    if type(string) is float: return string * u.s
    if string.endswith("min"):
        string = string.removesuffix("min")
        return float(string) * 60 * u.s 
    elif string.endswith("hr"): 
        string = string.removesuffix("hr")
        return float(string) * 3600 * u.s
    elif string.endswith("d"):
        string  = string.removesuffix("d")
        return float(string) * 86400 * u.s
    elif string.endswith("yr"):
        string = string.removesuffix("yr")
        return float(string) * 31536000 * u.s
    else: return float(string) * u.s

def array_to_vectors(array, unit):
    arr = []
    for a in array:
        arr.append(ax.to_vector(a, unit))
    return arr

def data_to_body(data):
    name = data["name"]
    mass = data["mass"] * u.kg
    
    if "x" in data["r"]:
        position = ax.to_vector(data["r"], u.m)
        velocity = ax.to_vector(data["v"], u.m/u.s)

        body = axtools.Body(name, mass, position, velocity, data["radius"] * u.m)

        if "color" in data:
            body.color = data["color"]
        if "light" in data:
            body.light = data["light"]
        if "retain" in data:
            body.retain = data["retain"]

        return body
    else:
        position = [vector_from_list(r, u.m) for r in data["r"].values()]
        velocity = [vector_from_list(v, u.m/u.s) for v in data["v"].values()]

        body = axtools.Body(name, mass, position[0], velocity[0], data["radius"] * u.m)

        for t, r in data["r"].items():
            body.r[to_float(t)] = vector_from_list(r, u.m)
        for t, v in data["v"].items():
            body.v[to_float(t)] = vector_from_list(v, u.m)

        if "color" in data:
            body.color = data["color"]
        if "light" in data:
            body.light = data["light"]
        if "retain" in data:
            body.retain = data["retain"]
        if "radius_multiplier" in data:
            body.radius *= data["radius_multiplier"]
        
        return body

def vector_from_list(vector: list, unit):
    return CartesianRepresentation(u.Quantity(float(vector[0]), unit), u.Quantity(float(vector[1]), unit), u.Quantity(float(vector[2]), unit))

def to_float(val):
    return np.float64(val)

def string_to_color(color_name, frontend: str):
    if frontend == "vpython":
        color_map = {
            'red': vp.color.red,
            'blue': vp.color.blue,
            'green': vp.color.green,
            'orange': vp.color.orange,
            'purple': vp.color.purple,
            'yellow': vp.color.yellow,
            'white': vp.color.white
        }
        return color_map.get(color_name, vp.color.white)
    elif frontend == "matplotlib":
        color_map = {
            'red': 'r',
            'blue': 'b',
            'green': 'g',
            'orange': 'orange',
            'purple': 'purple',
            'yellow': 'yellow',
            'white': 'white'
        }
        return color_map.get(color_name, 'white')


from axinite.tools.args import AxiniteArgs
from axinite.tools.body import Body
from axinite.tools.functions import *
from axinite.tools.load import load
from axinite.tools.read import read
from axinite.tools.show import show
from axinite.tools.live import live
from axinite.tools.frontends.vpython import vpython_frontend
from axinite.tools.frontends.plotly import plotly_frontend